﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Algorithms.ScatterSearch {
  /// <summary>
  /// An operator that updates the reference set.
  /// </summary>
  [Item("ReferenceSetUpdateMethod", "An operator that updates the reference set.")]
  [StorableClass]
  public sealed class ReferenceSetUpdateMethod : SingleSuccessorOperator {
    #region Parameter properties
    public ScopeParameter CurrentScopeParameter {
      get { return (ScopeParameter)Parameters["CurrentScope"]; }
    }
    public IValueLookupParameter<IntValue> ReferenceSetSizeParameter {
      get { return (IValueLookupParameter<IntValue>)Parameters["ReferenceSetSize"]; }
    }
    public IValueLookupParameter<SimilarityCalculator> SimilarityCalculatorParameter {
      get { return (IValueLookupParameter<SimilarityCalculator>)Parameters["SimilarityCalculator"]; }
    }
    public IValueLookupParameter<IItem> TargetParameter {
      get { return (IValueLookupParameter<IItem>)Parameters["Target"]; }
    }
    #endregion

    #region Properties
    private IScope CurrentScope {
      get { return CurrentScopeParameter.ActualValue; }
    }
    private IntValue ReferenceSetSize {
      get { return ReferenceSetSizeParameter.ActualValue; }
    }
    private SimilarityCalculator SimilarityCalculator {
      get { return SimilarityCalculatorParameter.ActualValue; }
    }
    private IItem Target {
      get { return TargetParameter.ActualValue; }
    }
    #endregion

    [StorableConstructor]
    private ReferenceSetUpdateMethod(bool deserializing) : base(deserializing) { }
    private ReferenceSetUpdateMethod(ReferenceSetUpdateMethod original, Cloner cloner) : base(original, cloner) { }
    public ReferenceSetUpdateMethod()
      : base() {
      #region Create parameters
      Parameters.Add(new ScopeParameter("CurrentScope"));
      Parameters.Add(new ValueLookupParameter<IntValue>("ReferenceSetSize"));
      Parameters.Add(new ValueLookupParameter<SimilarityCalculator>("SimilarityCalculator"));
      Parameters.Add(new ValueLookupParameter<IItem>("Target"));
      #endregion
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new ReferenceSetUpdateMethod(this, cloner);
    }

    public override IOperation Apply() {
      IDictionary<IScope, double> population = new Dictionary<IScope, double>();
      foreach (var pScope in CurrentScope.SubScopes[0].SubScopes) {
        double similarity = 0;
        foreach (var rScope in CurrentScope.SubScopes[1].SubScopes) {
          similarity += SimilarityCalculator.ExecuteCalculation(pScope, rScope);
        }
        population[pScope] = similarity;
      }
      int numberOfHighQualitySolutions = CurrentScope.SubScopes[1].SubScopes.Count;
      foreach (var entry in population.OrderBy(x => x.Value).Take(ReferenceSetSize.Value - numberOfHighQualitySolutions)) {
        CurrentScope.SubScopes[1].SubScopes.Add(entry.Key);
        CurrentScope.SubScopes[0].SubScopes.Remove(entry.Key);
      }
      return base.Apply();
    }
  }
}
